/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse, die __eine__ Frage auf dem Bildschirm darstellt und auswertet
*/
function FragenUI(
    data,       // Daten und Einstellungen dieser Frage
    nodeParent, // HTMLElement, in dem die Frage angezeigt werden soll
    intNumber   // Nummer der Frage
) {
    this.data = data;
    this._choices = [];
    this._establish(nodeParent, intNumber);
}

// Daten und Einstellungen dieser Frage
FragenUI.prototype.data = {};

/**
* Frage als HTML anlegen
* @return undefined
*/
FragenUI.prototype._establish = function(
    nodeParent, // HTMLElement, in dem die Frage angezeigt werden soll
    intNumber   // Nummer der Frage
) {
    var node = AudioFeedbackTask.createHtml('fieldset', nodeParent);
    this.node = node;
    AudioFeedbackTask.createHtml('legend', node, null, this.data.title ? this.data.title : ('Frage ' + intNumber));
    if (this.data.html)
        AudioFeedbackTask.createHtml('div', node).innerHTML = this.data.html;
    var choices = this.data.choices;
    if (!this.data.keepOrder)
        choices = choices.map(function(v) { return { v: v, rand: Math.random() }; }).sort(function(a, b) { return a.rand - b.rand; }).map(function(o) { return o.v; });
    if (choices.length) {
        var ul = AudioFeedbackTask.createHtml('ol', node);
        var label, input, li;
        for (var z = 0; z < choices.length; ++z) {
            this._establishChoice(ul, intNumber, z + 1, choices[z]);
        }
    }
}

/**
* HTML für Antwortmöglichkeit anlegen und Antwortmöglichkeit für Auswertung vormerken
* @return undefined
*/
FragenUI.prototype._establishChoice = function(
    nodeList,       // HTMLListElement für Antwortmöglichkeiten
    intNumber,      // Nummer der Frage
    choiceNumber,   // Nummer der Antwortmöglichkeit
    data            // Daten zur Antwortmöglichkeit
) {
    var li = AudioFeedbackTask.createHtml('li', nodeList);
    var label = AudioFeedbackTask.createHtml('label', li);
    var input = AudioFeedbackTask.createHtml('input', AudioFeedbackTask.createHtml('span', label), { "name": "antwort" + intNumber, "value": choiceNumber, "type": this.data.singlechoice ? 'radio' : 'checkbox' });
    if (data.selectalways) {
        input.checked = true;
        input.disabled = true;
    }
    var choice = AudioFeedbackTask.createHtml('div', label);
    choice.innerHTML = data.html;
    var korrekturhinweise = { input: input, correct: data.correct, response: data.response };
    // Ggf. Eingabefeld und Einheit ergänzen?
    if (data.type) {
        var atts = { "id": "q" + intNumber + 'c' + choiceNumber + 'input' };
        ['type', 'min', 'max', 'step', 'minlength', 'maxlength', 'size', 'inputmode', 'pattern', 'placeholder', 'value', 'title'].map(function(att) {
            if (data[att] !== undefined)
                atts[att] = data[att];
        });
        korrekturhinweise.value = AudioFeedbackTask.createHtml('input', choice, atts);
        korrekturhinweise.value.addEventListener('input', function() {
            input.checked = true;
        });
        label.setAttribute('for', atts.id);
        if (data.unit)
            AudioFeedbackTask.createHtml('span', choice).innerHTML = data.unit;
        korrekturhinweise.casesensitive = ((data.casesensitive === undefined) || data.casesensitive) ? true : false;
        korrekturhinweise.accept = (data.accept === undefined) ? [''] : ((data.accept instanceof Array) ? data.accept : [ data.accept ]);
    }
    this._choices.push(korrekturhinweise);
}

/**
* Ggf. einen Korrekturhinweis ergänzen
* @return undefined
*/
FragenUI.prototype.addHint = function(
    hintHTML,       // Hinweistext (HTML)
    boolFirst       // Optional: Hinweis als erstes Element einfügen
) {
    if (!hintHTML)
        return;
    AudioFeedbackTask.createHtml('div', this.node, { 'class': 'hints' }).innerHTML = hintHTML;
}

/**
* Ein Texteingabefeld (falls vorhanden) durch die Lösung und Korrektur ersetzen
* @return Boolean   War die Eingabe richtig?
*/
FragenUI._resolveValue = function(
    korrekturhinweise       // Korrekturhinweise für diese Antwortmöglichkeit
) {
    if (!korrekturhinweise.value)
        return true;
    var value = korrekturhinweise.value.value.split(new RegExp('\\s+')).filter(function(s) { return s.length; }).join(' ');
    var correct;
    if (korrekturhinweise.value.type == 'number')
        correct = (korrekturhinweise.accept.map(function(v) {
            return Math.abs(+v - value);
        }).reduce(function(a, b) { return Math.min(a, b); }) < (korrekturhinweise.value.step ? korrekturhinweise.value.step : 1));
    else
        correct = (korrekturhinweise.casesensitive ? korrekturhinweise.accept.indexOf(value) : korrekturhinweise.accept.map(function(s) { return s.toUpperCase(); }).indexOf(value.toUpperCase())) != -1;
    if (correct) {
        AudioFeedbackTask.createHtml('span', null, null, value, korrekturhinweise.value);
    } else {
        AudioFeedbackTask.createHtml('del', null, null, value, korrekturhinweise.value);
        if (korrekturhinweise.accept[0])
            AudioFeedbackTask.createHtml('ins', null, null, korrekturhinweise.accept[0], korrekturhinweise.value);
    }
    korrekturhinweise.value.parentNode.removeChild(korrekturhinweise.value);
    return correct;
}

/**
* Ein Eingabefeld durch die Lösung und Korrekturzeichen ersetzen
* @return Boolean   War die Antwort richtig?
*/
FragenUI._resolveInput = function(
    korrekturhinweise,      // Korrekturhinweise für diese Antwortmöglichkeit
    correct                 // Muss das Eingabefeld ausgewählt sein? (undefined für egal, true oder false)
) {
    var valueCorrect = FragenUI._resolveValue(korrekturhinweise);
    var input = korrekturhinweise.input;
    var user = input.checked;
    var src = 'nix.svg';
    var alt = '✔✘☐☒';
    var result = false;
    switch ((valueCorrect ? 0 : 8) + (user ? 4 : 0) + ((correct === true) ? 1 : ((correct === false) ? 0 : 2))) {
        case 1:
        case 9:
            src = 'dochx.svg';
            alt = '✘☐';
            break;
        case 4:
        case 12:
            src = 'dochnix.svg';
            alt = '✘☒';
            break;
        case 13:
        case 14:
            src = 'justx.svg';
            alt = ' ☒';
            break;
        case 5:
        case 6:
            src = 'x.svg';
            alt = '✔☒';
            result = true;
            break;
        case 0:
        case 2:
        case 8:
        case 10:
        default:
            src = 'nix.svg';
            alt = '✔☐';
            result = true;
            break;
    }
    AudioFeedbackTask.createHtml('img', null, { alt: alt, src: 'share/' + src }, undefined, input);
    input.parentNode.parentNode.className = result ? 'correct' : 'wrong';
    input.parentNode.removeChild(input);
    return result;
}

/**
* Eine Frage korrigieren, die nur die Auswahl einer Auswahlmöglichkeit erlaubt
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: 1, score: "Richtig ausgewählte Antwortmöglichkeiten" }
*/
FragenUI.prototype._evaluateSinglechoice = function() {
    var correct = false;
    this._choices.map(function(choice) {
        if (choice.input.checked && choice.correct)
            correct = true;
    });
    var egal = correct;
    for (var z = 0; z < this._choices.length; ++z) {
        if (this._choices[z].correct) {
            if (!FragenUI._resolveInput(this._choices[z], this._choices[z].correct ? true : false)) {
                this.addHint(this._choices[z].response);
                correct = false;
            }
        } else {
            FragenUI._resolveInput(this._choices[z], egal ? undefined : (this._choices[z].correct ? true : false)) || this.addHint(this._choices[z].response);
        }
    }
    return { "correct": correct, "count": 1, "score": (correct ? 1 : 0) };
}

/**
* Eine Frage korrigieren, die mehrfache Auswahl von Antwortmöglichkeiten erlaubt
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: "Anzahl an Antwortmöglichkeiten", score: "Richtig ausgewählte Antwortmöglichkeiten" }
*/
FragenUI.prototype._evaluateMultiplechoice = function() {
    var correct = true;
    var score = 0;
    for (var z = 0; z < this._choices.length; ++z) {
        if (FragenUI._resolveInput(this._choices[z], this._choices[z].correct ? true : false))
            ++score;
        else {
            correct = false;
            this.addHint(this._choices[z].response);
        }
    }
    return { "correct": correct, "count": this._choices.length, "score": score };
}

/**
* Frage korrigieren
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: "Erreichbare Punkteanzahl", score: "Erreichte Punkteanzahl" }
*/
FragenUI.prototype.evaluate = function() {
    var r = (this.data.singlechoice ? this._evaluateSinglechoice() : this._evaluateMultiplechoice());
    this.node.className = (r.correct ? 'correct' : 'wrong');
    if (!r.correct)
        this.addHint(this.data.response);
    return r;
}
